<?php

namespace App\Controller;

use App\Http\HttpRequest;
use App\Http\HttpResponse;
use App\Http\ViewHttpResponse;
use App\Model\TeamModel;
use App\Validation\FieldValidationFail;
use App\Validation\Validators;
use \Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;

class TeamController /* verif si les camp sont assignés, sinon erreur 400*/
{
    private TeamModel $model;
    private Environment $twig;

    /**
     * @param TeamModel $model
     * @param Environment $twig
     */
    public function __construct(TeamModel $model, Environment $twig) {
        $this->model = $model;
        $this->twig = $twig;
    }

    public function displaySubmitTeam() {
        try {
            $this->twig->display("insert_team.html.twig", []);
        } catch (LoaderError | RuntimeError | SyntaxError $e) {
            echo " twig error : $e";
        }
    }

    public function submitTeam(array $request): HttpResponse {

        $errors = [];
        $request = HttpRequest::from($request, $errors, [
            "name" => [Validators::lenBetween(1, 32), Validators::nameWithSpaces()],
            "mainColor" => [Validators::isInteger(), Validators::isIntInRange(0, 0xffffff)],
            "secondColor" => [Validators::isInteger(), Validators::isIntInRange(0, 0xffffff)],
            "picture" => [Validators::isURL()]
        ]);
        if (!empty($errors)) {
            $badFields = [];
            foreach ($errors as $e) {
                if ($e instanceof FieldValidationFail) {
                    $badFields[] = $e->getFieldName();
                }
            }
            return ViewHttpResponse::twig('insert_team.html.twig',['bad_fields'=> $badFields]);
        }
        $this->model->createTeam($request['name'], $request['picture'], intval($request['mainColor']), intval($request['secondColor']));
        return ViewHttpResponse::twig('sample_form.html.twig',[]); /*todo appeler une vue qui display la team au lieu de ça*/
    }

    public function displayListTeamByName(){
        try {
            $this->twig->display("list_team_by_name.html.twig", []);
        } catch (LoaderError | RuntimeError | SyntaxError $e) {
            echo " twig error : $e";
        }
    }
    public function listTeamByName(array $request): HttpResponse {
        $errors = [];
        $request = HttpRequest::from($request, $errors, [
                "name" => [Validators::lenBetween(1,32),Validators::nameWithSpaces()]
        ]);
        if(!empty($errors) && $errors[0] instanceof FieldValidationFail){
            $badField = $errors[0]->getFieldName();
            return ViewHttpResponse::twig('list_team_by_name.html.twig',['bad_field' => $badField]) ;
        }
        $results = $this->model->listByName($request['name']);
        if (empty($results)){
            /*todo appelle de la bonne vue qui va afficher un message qui dit que bah ca retourne rien, proposer de refaire une recherche*/
        }
        return ViewHttpResponse::twig('display_teams.html.twig',['teams' => $results]);
    }
}


